/**
 * Implement Gatsby's Node APIs in this file.
 *
 * See: https://www.gatsbyjs.com/docs/reference/config-files/gatsby-node/
 */

/**
 * @type {import('gatsby').GatsbyNode['createPages']}
 */
const path = require('path');
const { generateNavigationTree } = require('./src/utils/createRecursiveMenu');
const startCase = str => str.replace(/([A-Z])/g, ' $1').replace(/^./, str => str.toUpperCase());
function getLastSegment(url) {
  const segments = url.split('/');

  if (segments[segments.length - 1] === 'index.mdx') {
    if (segments.length > 1) {
      return `${segments[segments.length - 2]}/${segments[segments.length - 1]}`;
    }
  }

  return segments[segments.length - 1];
}

exports.createPages = async ({ actions, graphql, reporter, cache }) => {
  const result = await graphql(
    `
      {
        site {
          siteMetadata {
            title
            description
            themeConfig {
              useLegacyUrls
              showFullNavigation
            }
          }
        }
        allMdx(filter: {fields: {source: {eq: "content"}}}) {
          edges {
            node {
              fields {
                id
                title
                slug
              }
              internal {
                contentFilePath
              }
            }
          }
        }
      }
    `
  );

  if (result.errors) {
    reporter.panic('error loading content', result.errors);
    return;
  }

  await cache.set('docskit_site_allmdx_data', result);

  const docsTemplate = path.join(__dirname, 'src/templates/docs.js');
  result.data.allMdx.edges.forEach(({ node }) => {
    actions.createPage({
      path: node.fields.slug ? node.fields.slug : '/',
      component: `${docsTemplate}?__contentFilePath=${node.internal.contentFilePath}`,
      context: {
        id: node.fields.id
      }
    });
  });
};

exports.onCreatePage = async ({ page, actions, cache }, themeOptions) => {
  const { createPage, deletePage } = actions

  const { useLegacyUrls = false, showFullNavigation = false, basePath } = themeOptions;
  const docsKitSiteAllMdxResults = await cache.get('docskit_site_allmdx_data');

  if (!docsKitSiteAllMdxResults) {
    reporter.panic('error loading content from cache');
    return;
  }

  const routes = docsKitSiteAllMdxResults.data.allMdx.edges.map(({ node }) => ({ ...node.fields, fileName: getLastSegment(node.internal.contentFilePath) }));
  const treeData = generateNavigationTree(routes);

  deletePage(page);

  createPage({
    ...page,
    context: {
      ...page.context,
      navigationTree: treeData,
      metaData: {
        ...docsKitSiteAllMdxResults.data.site.siteMetaData,
        themeConfig: {
          useLegacyUrls,
          showFullNavigation,
          basePath
        }
      }
    },
  })
};

exports.onCreateNode = ({ node, getNode, actions, reporter }, themeOptions) => {
  const { createNodeField } = actions;
  const { useLegacyUrls = false } = themeOptions;

  if (node.internal.type === 'Mdx') {
    const parent = getNode(node.parent);
    const title = node.frontmatter.title || startCase(parent.name)

    let value = node.frontmatter.slug;
    if (!value && parent.relativePath) {
      value = parent.relativePath.replace(parent.ext, '');
    }

    if (!value) {
      reporter.panic(`Can not create node with title: ${title} there is no relative path or frontmatter to set the "slug" field`);
      return;
    }

    let slug = `/${value}`
    if (useLegacyUrls) {
      slug = `/${value}/index.html`
    }

    if (value === 'index') {
      slug = '/';
    }

    createNodeField({
      name: 'slug',
      node,
      value: slug
    });

    createNodeField({
      name: 'id',
      node,
      value: node.id
    });

    createNodeField({
      name: 'title',
      node,
      value: title
    });
  }
};

exports.createSchemaCustomization = ({ actions }) => {
  const { createTypes } = actions

  createTypes(`
    type Mdx implements Node {
      frontmatter: MdxFrontmatter!
    }
    type MdxFrontmatter {
      hideNavigation: Boolean
      hideToc: Boolean
    }
  `)
}

exports.pluginOptionsSchema = ({ Joi }) => {
  return Joi.object({
    useLegacyUrls: Joi.boolean().description(`Use legacy URLs with /index.html`),
    showFullNavigation: Joi.boolean().description(`Always show full navigation on every page`),
    basePath: Joi.string().description(`Site basePath (same as pathPrefix in gatsby-config.js)`)
  });
};
